#include "Arduino.h"
#include "SPI.h"
#include "ad1247.h"

/*
 * AD関係pin初期化
 */
void ad_pin_init()
{
	pinMode(AD_DRDY, INPUT);
	pinMode(AD_CS, OUTPUT);
	pinMode(AD_START, OUTPUT);

	ad_cs_dis();
	ad_start_dis();
}
/**
 * AD RESETのみ
 */
void ad_reset()
{
	// START=Hでないとコマンドを受け付けない
	ad_start_ena();

	ad_cs_ena();
		SPI.transfer(RESET); //Reset the ADS1247
	ad_cs_dis();
	delay(5); //Minimum 0.6ms required for Reset to finish.
}
/**
 * @brief AD測定前の基本設定
 */
void ad_init()
{
	// START=Hでないとコマンドを受け付けない
	ad_start_ena();
	ad_cs_ena();
		SPI.transfer(WREG | 0x00); //Set MUX0 Register (00h)
		SPI.transfer(0x00);
		SPI.transfer(0x01);	// 電流源OFF, AIN0:+IN, AIN1:-IN
	ad_cs_dis();

	ad_cs_ena();
		SPI.transfer(WREG | 0x02); //Set MUX1 Register (02h)
		SPI.transfer(0x00);
		SPI.transfer(0x30);	// 内蔵VREF常時ON, 内蔵VREF使用, 通常動作
//SPI.transfer(0x33);	// 内蔵VREF常時ON, 内蔵VREF使用, 温度測定

//		SPI.transfer(0x50);	// 内蔵VREF変換中のみON, 内蔵VREF使用, 通常動作
//SPI.transfer(0x51);	// 内蔵VREF変換中のみON, 内蔵VREF使用, 入力を電源中点に固定
//SPI.transfer(0x36);	// 内蔵VREF常時ON, 内蔵VREF使用, AVDD測定
//SPI.transfer(0x3E);	// 内蔵VREF常時ON, 内蔵VREF使用 REF0に接続, AVDD測定
//SPI.transfer(0x20);	// 内蔵VREF常時ON

	ad_cs_dis();

	ad_cs_ena();
		SPI.transfer(WREG | 0x03); //Set SYS0 Register (03h)
		SPI.transfer(0x00);
		SPI.transfer(0x00);	// PGA=1, Rate=5Hz
	ad_cs_dis();
}
/**
 * @brief AD CH切換えのみ
 * @param ch 0=CH1, 1=CH2
 * 
 */ 
void ad_conf_ch(uint8_t ch)
{
	// START=Hでないとコマンドを受け付けない
	ad_start_ena();
	ad_cs_ena();
		SPI.transfer(WREG | 0x00); //Set MUX0 Register (00h)
		SPI.transfer(0x00);
		if (ch == 0) {
			SPI.transfer(0x01);	// 電流源OFF, AIN0:+IN, AIN1:-IN
		} else {
			SPI.transfer(0x13);	// 電流源OFF, AIN2:+IN, AIN3:-IN
		}
	ad_cs_dis();
}

/**
 * @brief AD変換値Get
 * @return AD変換値 2の補数
 */
int32_t	ad_get_val()
{
	uint32_t	A2DVal;

	A2DVal = 0;
	ad_cs_ena();
		SPI.transfer(RDATA); //Issue RDATA
		A2DVal |= SPI.transfer(0xFF); //Write NOP, Read First Byte and Mask to A2DVal
		A2DVal <<= 8; //Left Bit-Shift A2DVal by 8 bits
		A2DVal |= SPI.transfer(0xFF); //Write NOP, Read Second Byte and Mask to A2DVal
		A2DVal <<= 8;
		A2DVal |= SPI.transfer(0xFF); //Write NOP, Read Third Byte and Mask to A2DVal
	ad_cs_dis();
	if (A2DVal & 0x00800000) {
		A2DVal |= 0xFF000000;
	}
	return A2DVal;
}
